@@ -12,3 +12,7 @@ class DateModelSpeed(MagicModelSpeed):
     def update(self, last_updated=None, *args, **kwargs):
         self.last_updated = last_updated or datetime.utcnow()
         return super().update(*args, **kwargs)
+
+    def save(self, last_updated=None, *args, **kwargs):
+        self.last_updated = last_updated or datetime.utcnow()
+        return super().save(*args, **kwargs)
