@@ -12,7 +12,6 @@ from glom import glom
 
 from magicdb.Timing.decorator import timing
 
-
 from pydantic import BaseModel, Field, PrivateAttr
 from pydantic.main import ModelMetaclass
 
@@ -29,6 +28,8 @@ from magicdb.Models.MagicCache import MagicCache
 
 import redis
 
+from magicdb.Models.RedisModel import RedisModel
+
 
 class DatabaseError(Exception):
     def __init__(self, key):
@@ -50,11 +51,51 @@ class QueryMeta(type):
         return QuerySpeed(cls).collection_group()
 
 
-class QueryAndBaseMetaClasses(ModelMetaclass, QueryMeta):
+class MetaBase:
+    collection_name: str
+    triggers: list = []
+    update_algolia_triggers: list = []
+    default_exception: Exception = Exception
+    redis_model: RedisModel = None
+    _set_meta: bool = False
+
+
+class CreateMetaClass(type):
+    """I am using a metaclass here because Student should have a fully fledged Meta class without being instantiated."""
+
+    def __new__(mcs, name, bases, dct):
+        class Meta(MetaBase):
+            collection_name: str = name.lower()
+
+        new_meta = Meta
+        x = super().__new__(mcs, name, bases, dct)
+
+        if given_meta := dct.get("Meta"):
+            new_meta._set_meta = True
+            given_fields = [f for f in dir(given_meta) if not f.startswith("__")]
+            for field in given_fields:
+                setattr(new_meta, field, getattr(given_meta, field))
+            x.Meta = new_meta
+            return x
+
+        inherited_meta = getattr(x, "Meta", None)
+        if not inherited_meta:
+            x.Meta = new_meta
+        elif inherited_meta._set_meta is False:
+            # it was never set, so use collection_name of curr cls
+            x.Meta = new_meta
+
+        return x
+
+
+class QueryAndBaseMetaClasses(CreateMetaClass, ModelMetaclass, QueryMeta):
     pass
 
 
-class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
+from magicdb.Models.BaseModelName import BaseModelName
+
+
+class MagicModelSpeed(BaseModelName, metaclass=QueryAndBaseMetaClasses):
     """
     When this gets inited, if given an id or key, assign based on that.
     Otherwise, assign them based on what Firestore gives it
@@ -74,12 +115,24 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
 
     @timing
     def __init__(self, from_db: bool = False, **data):
+        self.Meta: T.Type[MetaBase]
+
         data_from_db = deepcopy(data)
+
+        temp_id_key = "__temp__"
+        if not data.get("id"):
+            data["id"] = temp_id_key
+
         super().__init__(**data)
 
-        FieldMaker(
-            **data, _collection_name=self.get_collection_name()
-        ).set_model_fields(self)
+        if self.id == temp_id_key:
+            self.id = None
+        if data.get("id") == temp_id_key:
+            del data["id"]
+
+        FieldMaker(**data, _collection_name=self.Meta.collection_name).set_model_fields(
+            self
+        )
 
         self._data_from_db = {} if not from_db else data_from_db
 
@@ -188,14 +241,6 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
             key_values.append(f"{repr(field)}={repr(getattr(self, field, None))}")
         print(join_str.join(key_values))
 
-    """META"""
-
-    class Meta:
-        """Init the meta class so you can use it and know it is there"""
-
-        """Looks like having Meta in the TestModel actually overwrites this one, not inherits it"""
-        ...
-
     @property
     def collection_name(self) -> str:
         return self.get_collection_name()
@@ -206,10 +251,12 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
 
     @classmethod
     def get_collection_name(cls) -> str:
-        return getattr(cls.Meta, "collection_name", cls.make_default_collection_name())
+        return cls.Meta.collection_name
 
     @classmethod
     def get_redis(cls) -> T.Optional[redis.Redis]:
+        if redis_model := cls.Meta.redis_model:
+            return redis_model.r
         if r := getattr(cls.Meta, "redis", None):
             if getattr(cls.Meta, "use_redis_span", True):
                 error_thrower = getattr(cls.Meta, "redis_error_thrower", None)
@@ -218,6 +265,8 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
 
     @classmethod
     def get_redis_ttl_secs(cls) -> T.Optional[int]:
+        if redis_model := cls.Meta.redis_model:
+            return redis_model.ttl_secs
         return getattr(cls.Meta, "redis_ttl_secs", None)
 
     @classmethod
@@ -332,6 +381,18 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
 
         return self
 
+    def make_update_d(self):
+        new_d = self.dict()
+        new_d = self.remove_magic_fields(new_d)  # must remove id
+        data_from_db = self.remove_magic_fields(self._data_from_db)
+
+        if not data_from_db:
+            update_d = new_d
+        else:
+            update_d = make_update_obj(original=data_from_db, new=new_d)
+
+        return update_d
+
     @magic_async
     @timing
     def update(
@@ -348,14 +409,7 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
         if not ignore_fields:
             self.validate_all_fields()
 
-        new_d = self.dict()
-        new_d = self.remove_magic_fields(new_d)  # must remove id
-        data_from_db = self.remove_magic_fields(self._data_from_db)
-
-        if not data_from_db:
-            update_d = new_d
-        else:
-            update_d = make_update_obj(original=data_from_db, new=new_d)
+        update_d = self.make_update_d()
 
         if print_update_d or only_print_update_d:
             pprint.pprint({f"update_d_for_{self.id}": update_d})
@@ -521,7 +575,7 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
 
     @classmethod
     def get_triggers(cls) -> list:
-        return getattr(cls.Meta, "triggers", None)
+        return cls.Meta.triggers
 
     @staticmethod
     def get_class_levels(cls: type(MagicModelSpeed), level: int) -> T.List[ClassLevel]:
@@ -536,7 +590,7 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
     @staticmethod
     def get_trigger_class_from_collection_name(
         collection_name: str,
-    ) -> type(MagicModelSpeed):
+    ) -> T.Type[MagicModelSpeed]:
         class_levels = MagicModelSpeed.get_class_levels(cls=MagicModelSpeed, level=1)
         matches = []
         for class_level in class_levels:
@@ -548,6 +602,44 @@ class MagicModelSpeed(BaseModel, metaclass=QueryAndBaseMetaClasses):
         if matches:
             return max(matches).cls
 
+    @staticmethod
+    def get_all_collection_names_with_triggers() -> T.Set[str]:
+        subs = MagicModelSpeed.get_all_subclasses_of_model()
+        collection_names = set()
+        for sub in subs:
+            if sub.get_triggers():
+                collection_names.add(sub.get_collection_name())
+        return collection_names
+
+    @staticmethod
+    def get_collections_with_triggers() -> T.List[T.Type[MagicModelSpeed]]:
+        names = MagicModelSpeed.get_all_collection_names_with_triggers()
+        return [
+            MagicModelSpeed.get_trigger_class_from_collection_name(collection_name=name)
+            for name in names
+        ]
+
+    @staticmethod
+    def refresh_all_triggers() -> None:
+        start = time.time()
+        classes = MagicModelSpeed.get_collections_with_triggers()
+        print(f"Refreshing triggers in {len(classes)} classes: {classes}.")
+        for cls in classes:
+            cls.refresh_triggers()
+        print(
+            f"Took {time.time() - start} to refresh all triggers in all {len(classes)} classes."
+        )
+
+    @classmethod
+    def refresh_triggers(cls):
+        start = time.time()
+        triggers = cls.get_triggers()
+        print(f"Refreshing {len(triggers)} triggers for {cls.__name__}.")
+        for trigger in triggers:
+            print(f"Refreshing trigger {trigger.__name__}.")
+            trigger.refresh(model_cls=cls)
+        print(f"Took {time.time() - start} to refresh all {cls.__name__} triggers.")
+
 
 class ClassLevel(BaseModel):
     cls: type(MagicModelSpeed)
