@@ -1,8 +1,12 @@
+from magicdb.Models.BaseModelName import BaseModelName
 from magicdb.Models.MagicCache import MagicCache
 from magicdb.Models.MagicModel import MagicModel
-from magicdb.Models.MagicModelSpeed import MagicModelSpeed
+from magicdb.Models.RedisModel import RedisModel
+from magicdb.Models.MagicModelSpeed import MagicModelSpeed, MetaBase
 from magicdb.Models.DateModel import DateModel
 from magicdb.Models.DateModelSpeed import DateModelSpeed
 from magicdb.Models.FrontendParserModel import FrontendParserModel
 from magicdb.Models.RemoveFields import RemoveFieldsMeta
 from magicdb.Models.RemoveFieldsSpeed import RemoveFieldsMetaSpeed
+from magicdb.Models.ReadOnlyModel import ReadOnlyModel
+from magicdb.Models.computed_fields import computed_fields
